import logging
import unittest
from io import StringIO
import sys

from EasyLoggerAJM.handlers import StreamHandlerIgnoreExecInfo


class TestStreamHandlerIgnoreExecInfo(unittest.TestCase):

    def setUp(self):
        # Set up a StreamHandlerIgnoreExecInfo with a StringIO as its stream.
        self.stream = StringIO()
        self.handler = StreamHandlerIgnoreExecInfo(self.stream)

    def tearDown(self):
        # Close the StringIO stream and the handler after each test.
        self.stream.close()
        self.handler.close()

    def test_emit_without_exc_info(self):
        # If the record has no exc_info, the message should still be logged normally.
        record = logging.LogRecord("my_logger", logging.INFO, "dummy_path", 0, "Hello, world!", None, None)
        self.handler.emit(record)
        self.assertEqual(self.stream.getvalue(), "Hello, world!\n")

    def test_emit_with_exc_info(self):
        # If the record has exc_info, it should be temporarily removed before logging.
        try:
            raise Exception("This is a dummy exception")
        except Exception:
            record = logging.LogRecord("my_logger", logging.ERROR, "dummy_path", 0,
                                       "An error has occurred!", None, sys.exc_info())
        self.handler.emit(record)
        self.assertNotIn("This is a dummy exception", self.stream.getvalue())

    def test_emit_restores_exc_info(self):
        # exc_info should be restored to the record after logging.
        try:
            raise Exception("This is another dummy exception")
        except Exception:
            record = logging.LogRecord("my_logger", logging.ERROR, "dummy_path", 0,
                                       "Another error has occurred!", None, sys.exc_info())
            old_exc_info = record.exc_info
            self.handler.emit(record)
            self.assertEqual(old_exc_info, record.exc_info)


if __name__ == '__main__':
    unittest.main()


from logging import Formatter

NO_COLORIZER = False
try:
    from ColorizerAJM import Colorizer
except (ModuleNotFoundError, ImportError):
    NO_COLORIZER = True


class ColorizedFormatter(Formatter):
    """
    Class that extends logging.Formatter to provide colored output based on log level.
    It includes methods to format log messages and exceptions with colors specified for
     warnings, errors, and other log levels.
    """
    def __init__(self, fmt=None, datefmt=None, style='%', validate=True):
        super().__init__(fmt, datefmt, style, validate)
        if NO_COLORIZER:
            return
        else:
            self.colorizer = Colorizer()

        self.debug_color = self.colorizer.__class__.LIGHT_GRAY
        self.info_color = self.colorizer.__class__.WHITE
        self.warning_color = self.colorizer.__class__.YELLOW
        self.error_color = self.colorizer.__class__.RED
        self.other_color = self.colorizer.__class__.GRAY

    def _get_record_color(self, record):
        if record.levelname == "WARNING":
            return self.warning_color
        elif record.levelname == "ERROR":
            return self.error_color
        elif record.levelname == "DEBUG":
            return self.debug_color
        elif record.levelname == "INFO":
            return self.info_color
        elif record.levelname == "CRITICAL":
            return self.error_color
        else:
            return self.other_color

    def formatMessage(self, record):
        if NO_COLORIZER:
            return super().formatMessage(record)
        else:
            return self.colorizer.colorize(text=super().formatMessage(record),
                                           color=self._get_record_color(record), bold=True)

    def formatException(self, ei):
        if NO_COLORIZER:
            return super().formatException(ei)
        else:
            return self.colorizer.colorize(text=super().formatException(ei),
                                           color=self._get_record_color(ei), bold=True)



import unittest
import re

from EasyLoggerAJM.easy_logger import EasyLogger
from EasyLoggerAJM.easy_logger import _EasyLoggerCustomLogger
from logging import getLogger, Logger
from test_EasyLoggerCustomLogger import TestEasyLoggerCustomLogger


# noinspection PyStatementEffect
class TestEasyLogger(unittest.TestCase):
    def setUp(self):
        test_attrs = {"project_name": "TestProject", "root_log_location": "./test_logs"}

        self.easy_logger_default = EasyLogger(** test_attrs)
        self.test_dir = self.easy_logger_default._root_log_location
        self.default_logger = self.easy_logger_default.logger

        self.easy_logger_non_default = EasyLogger(** test_attrs,
                                                  logger=getLogger())
        self.non_default_logger = self.easy_logger_non_default.logger

    def test_creation(self):
        self.assertIsInstance(self.easy_logger_default, EasyLogger)

    def test_logger_inst_creation(self):
        self.assertIsInstance(self.default_logger, _EasyLoggerCustomLogger)

    def test_logger_nonCustomLogger_inst_creation(self):
        self.assertNotIsInstance(self.non_default_logger, _EasyLoggerCustomLogger)
        self.assertIsInstance(self.non_default_logger, Logger)

    def test_project_name(self):
        self.assertEqual(self.easy_logger_default.project_name, "TestProject")

    def test_default_format(self):
        self.assertEqual(self.easy_logger_default.DEFAULT_FORMAT, '%(asctime)s | %(name)s | %(levelname)s | %(message)s')

    def test_inner_log_fstructure(self):
        self.assertIsNotNone(self.easy_logger_default.inner_log_fstructure)

    def test_log_location(self):
        posix_with_leading_dir = ('./' + self.easy_logger_default.log_location.as_posix())
        self.assertTrue(re.match(f"{self.test_dir}.*", posix_with_leading_dir))

    def test_useLogger_creation(self):
        logger_cl = EasyLogger(project_name="TestProject2", root_log_location=f"{self.test_dir}2")
        logger = EasyLogger.UseLogger()
        self.assertIsInstance(logger, _EasyLoggerCustomLogger)
        self.assertEqual(logger_cl.project_name, "TestProject2")

        posix_with_leading_dir = ('./' + logger_cl.log_location.as_posix())
        self.assertTrue(re.match(f"{self.test_dir}2.*", posix_with_leading_dir))

    def test_make_file_handlers(self):
        self.easy_logger_default.make_file_handlers()
        self.assertIsNotNone(self.default_logger.handlers)
        self.assertGreaterEqual(len(self.default_logger.handlers), len(self.easy_logger_default.file_logger_levels))

    def test_logger_level_normalization_with_kwargs(self):
        self.easy_logger_default = EasyLogger(project_name="TestProject", root_log_location="./test_logs", file_logger_levels=['DEBUG', 'INFO', 'WARNING', 'ERROR'])
        self.easy_logger_default.make_file_handlers()
        for x in self.easy_logger_default.file_logger_levels:
            self.assertIn(x, [x.level for x in self.default_logger.handlers])
            self.assertIsInstance(x, int)

    def test_is_daily_log_spec(self):
        dls_logger = EasyLogger(project_name="TestProject3",
                                root_log_location=f"{self.test_dir}3",
                                is_daily_log_spec=True)
        dls_logger.make_file_handlers()
        self.assertEqual(dls_logger.inner_log_fstructure.split('/')[0], dls_logger.DAILY_LOG_SPEC_FORMAT)

    def test_given_dictionary_when_getting_log_spec_then_return_value(self):
        self.easy_logger_default.log_spec = {'name': 'minute'}
        self.assertEqual(self.easy_logger_default.log_spec, self.easy_logger_default.LOG_SPECS['minute'])

    def test_given_incorrect_key_dictionary_when_getting_log_spec_then_raise_exception(self):
        with self.assertRaises(KeyError):
            self.easy_logger_default.log_spec = {'wrong_key': 'minute'}

    def test_given_string_when_getting_log_spec_then_return_value(self):
        self.easy_logger_default.log_spec = 'minute'
        self.assertEqual(self.easy_logger_default.log_spec, self.easy_logger_default.LOG_SPECS['minute'])

    def test_given_wrong_string_when_getting_log_spec_then_raise_exception(self):
        with self.assertRaises(AttributeError):
            self.easy_logger_default.log_spec = 'wrong_string'

    def test_given_wrong_case_string_when_getting_log_spec_then_make_lowercase(self):
        self.easy_logger_default.log_spec = 'Minute'
        self.assertTrue(self.easy_logger_default.log_spec['name'].islower())

    def test_given_wrong_case_string_in_dict_when_getting_log_spec_then_make_lowercase(self):
        self.easy_logger_default.log_spec = {'name': 'Minute'}
        self.assertTrue(self.easy_logger_default.log_spec['name'].islower())

    def test_given_none_when_getting_log_spec_then_return_default_value(self):
        self.easy_logger_default.log_spec = None
        self.assertEqual(self.easy_logger_default.log_spec, self.easy_logger_default.LOG_SPECS['minute'])


if __name__ == "__main__":
    unittest.main()
    TestEasyLoggerCustomLogger.tearDownClass()




import logging
from abc import abstractmethod
from datetime import datetime
from pathlib import Path
from typing import Union, Optional

from EasyLoggerAJM import ConsoleOneTimeFilter


class _LogSpec:
    """
        Class `_LogSpec` is a container for predefined log specifications and timestamp formats,
        organized by time intervals (daily, hourly, and minute-based).
        These specifications are auto-generated based on the current date and time.

        Attributes:
            MINUTE_LOG_SPEC_FORMAT : tuple
                A tuple containing the current date (ISO formatted) and time up to minutes in string format without colons.
            MINUTE_TIMESTAMP : str
                A compact ISO formatted timestamp up to minutes excluding colons.

            HOUR_LOG_SPEC_FORMAT : tuple
                A tuple containing the current date (ISO formatted) and time truncated to the hour in string format
                    (e.g., "1400" for 2 PM).
            HOUR_TIMESTAMP : str
                A compact ISO formatted timestamp indicating the hour without colons.

            DAILY_LOG_SPEC_FORMAT : str
                The current date as an ISO formatted string.
            DAILY_TIMESTAMP : str
                The current date truncated to the hour component formatted in ISO standard.

            LOG_SPECS : dict
                A dictionary containing log specifications for daily, hourly, and minute time intervals.
                    Each key corresponds to the time interval ('daily', 'hourly', 'minute') and its value is another dictionary with:
                    - 'name': Name of the log interval.
                    - 'format': Predefined time format of the given interval.
                    - 'timestamp': Compact timestamp matching the logical interval.
    """

    # TODO: replace these with checks using logging.getlevelname()
    INT_TO_STR_LOGGER_LEVELS = {
        10: 'DEBUG',
        20: 'INFO',
        30: 'WARNING',
        40: 'ERROR',
        50: 'CRITICAL'
    }

    STR_TO_INT_LOGGER_LEVELS = {
        'DEBUG': 10,
        'INFO': 20,
        'WARNING': 30,
        'ERROR': 40,
        'CRITICAL': 50
    }

    # this is a tuple of the date and the time down to the minute
    MINUTE_LOG_SPEC_FORMAT = (datetime.now().date().isoformat(),
                              ''.join(datetime.now().time().isoformat().split('.')[0].split(":")[:-1]))
    MINUTE_TIMESTAMP = datetime.now().isoformat(timespec='minutes').replace(':', '')

    HOUR_LOG_SPEC_FORMAT = datetime.now().date().isoformat(), (
            datetime.now().time().isoformat().split('.')[0].split(':')[0] + '00')
    HOUR_TIMESTAMP = datetime.now().time().isoformat().split('.')[0].split(':')[0] + '00'

    DAILY_LOG_SPEC_FORMAT = datetime.now().date().isoformat()
    DAILY_TIMESTAMP = datetime.now().isoformat(timespec='hours').split('T')[0]

    LOG_SPECS = {
        'daily': {
            'name': 'daily',
            'format': DAILY_LOG_SPEC_FORMAT,
            'timestamp': DAILY_TIMESTAMP
        },
        'hourly': {
            'name': 'hourly',
            'format': HOUR_LOG_SPEC_FORMAT,
            'timestamp': HOUR_TIMESTAMP
        },
        'minute': {
            'name': 'minute',
            'format': MINUTE_LOG_SPEC_FORMAT,
            'timestamp': MINUTE_TIMESTAMP
        }
    }


# noinspection PyUnresolvedReferences
class _InternalLoggerMethods:
    """
    This class contains internal utility methods for configuring and logging internal
    operations of the logger. These methods are designed for internal use and handle
    logging of initial attributes, setting up file and stream handlers, and initializing
    the internal logger.

    Methods
    -------
    _log_attributes_internal(logger_kwargs)
        Logs the initial state of key instance attributes and any additional keyword
        arguments passed during initialization.

    _setup_internal_logger_handlers(verbose=False)
        Sets up handlers for the internal logger, including a file handler to log into
        a predefined file and, optionally, a stream handler for console output.

    _setup_internal_logger(**kwargs)
        Initializes and configures the internal logger with a designated logging level
        and handlers. Returns the initialized logger.
    """

    def _log_attributes_internal(self, logger_kwargs):
        """
        Logs internal attributes and initialization parameters for debugging purposes.

        :param logger_kwargs: Arguments passed during the initialization of the instance.
        :type logger_kwargs: dict
        """
        self._internal_logger.info(f"root_log_location set to {self._root_log_location}")
        self._internal_logger.info(f"chosen_format set to {self._chosen_format}")
        self._internal_logger.info(f"no_stream_color set to {self._no_stream_color}")
        self._internal_logger.info(f"kwargs passed to __init__ are {logger_kwargs}")

    def _setup_internal_logger_handlers(self, verbose=False):
        """
        :param verbose: Indicates whether to enable verbose logging. If True, adds a StreamHandler to log messages to the console.
        :type verbose: bool
        :return: None
        :rtype: None
        """
        log_file_path = Path(self._root_log_location,
                             'EasyLogger_internal.log'.replace('\\', '/'))
        fmt = logging.Formatter(self._chosen_format)

        log_file_mode = 'w'
        if not log_file_path.exists():
            Path(self._root_log_location).mkdir(parents=True, exist_ok=True)
        h = logging.FileHandler(log_file_path, mode=log_file_mode)
        h.setFormatter(fmt)
        self._internal_logger.addHandler(h)

        if verbose:
            h2 = logging.StreamHandler()
            h2.setFormatter(fmt)
            self._internal_logger.addHandler(h2)

    def _setup_internal_logger(self, **kwargs):
        """
        Sets up the internal logger for the application.

        :param kwargs: Optional keyword arguments for configuring the logger.
            The key 'verbose' can be used to enable or disable verbose logging.
        :return: The configured internal logger instance.
        :rtype: logging.Logger
        """
        self._internal_logger = logging.getLogger('EasyLogger_internal')
        self._internal_logger.propagate = False
        self._internal_logger.setLevel(10)
        self._setup_internal_logger_handlers(verbose=kwargs.get('verbose', False))

        self._internal_logger.info("internal logger initialized")
        return self._internal_logger


class _PropertiesInitializer(_LogSpec):
    ROOT_LOG_LOCATION_DEFAULT = '../logs'

    def __init__(self, root_log_location=None):
        self._file_logger_levels = None
        self._project_name = None
        self._log_spec = None
        # noinspection SpellCheckingInspection
        self._inner_log_fstructure = None
        self._log_location = None
        self._root_log_location = root_log_location
        if self._root_log_location is None:
            self._root_log_location = self.__class__.ROOT_LOG_LOCATION_DEFAULT

    @classmethod
    def get_default_file_logger_levels(cls):
        return [cls.STR_TO_INT_LOGGER_LEVELS["DEBUG"],
                cls.STR_TO_INT_LOGGER_LEVELS["INFO"],
                cls.STR_TO_INT_LOGGER_LEVELS["ERROR"]]

    def _set_initial_properties_value(self, **kwargs):
        # properties
        self.file_logger_levels = kwargs.get('file_logger_levels', [])
        self.project_name = kwargs.get('project_name', None)
        self.log_spec = kwargs.get('log_spec', None)

    def _validate_file_logger_levels(self, fll: list):
        if [x for x in fll
            if x in self.__class__.STR_TO_INT_LOGGER_LEVELS
               or x in self.__class__.INT_TO_STR_LOGGER_LEVELS]:
            if any([isinstance(x, str) and not x.isdigit() for x in fll]):
                fll = [self.__class__.STR_TO_INT_LOGGER_LEVELS[x] for x in
                       fll]
            elif any([isinstance(x, int) for x in fll]):
                pass
        return fll

    @property
    def file_logger_levels(self):
        if not self._file_logger_levels:
            self._file_logger_levels = self.get_default_file_logger_levels()
        return self._file_logger_levels

    @file_logger_levels.setter
    def file_logger_levels(self, value):
        if not value:
            self._file_logger_levels = self.get_default_file_logger_levels()
        else:
            self._file_logger_levels = self._validate_file_logger_levels(value)
        self._internal_logger.info(f"file_logger_levels set to {self._file_logger_levels}")

    @property
    def project_name(self):
        """
        Getter for the project_name property.

        Returns the name of the project. If the project name has not been set previously,
         it is determined based on the filename of the current file.

        Returns:
            str: The name of the project.
        """
        return self._project_name

    @project_name.setter
    def project_name(self, value):
        self._project_name = value
        if not self._project_name:
            self._project_name = __file__.split('\\')[-1].split(".")[0]
        self._internal_logger.info(f"project_name set to {self._project_name}")

    # noinspection SpellCheckingInspection
    @property
    def inner_log_fstructure(self):
        """
        Getter method for retrieving the inner log format structure.

        This method checks the type of the log_spec['format'] attribute and returns
            the inner log format structure accordingly.
        If the log_spec['format'] is of type str, the inner log format structure is set as
            "{}".format(self.log_spec['format']).
        If the log_spec['format'] is of type tuple, the inner log format structure is set as
            "{}/{}".format(self.log_spec['format'][0], self.log_spec['format'][1]).

        Returns:
            str: The inner log format structure.
        """
        if isinstance(self.log_spec['format'], str):
            self._inner_log_fstructure = "{}".format(self.log_spec['format'])
        elif isinstance(self.log_spec['format'], tuple):
            self._inner_log_fstructure = "{}/{}".format(self.log_spec['format'][0], self.log_spec['format'][1])
        return self._inner_log_fstructure

    @property
    def log_location(self) -> Path:
        """
        Getter method for retrieving the log_location property.

        Returns:
            str: The absolute path of the log location.
        """
        self._log_location = Path(self._root_log_location,
                                  self.inner_log_fstructure)
        if self._log_location.is_dir():
            pass
        else:
            self._log_location.mkdir(parents=True, exist_ok=True)
        return self._log_location

    @property
    def log_spec(self):
        return self._log_spec

    @log_spec.setter
    def log_spec(self, value):
        if value is None:
            value = self.LOG_SPECS['minute']
        if isinstance(value, dict):
            try:
                self._log_spec = self.LOG_SPECS[value['name'].lower()]
            except KeyError:
                raise KeyError("if log_spec is given as a dictionary, "
                               "it must include the key/value for 'name'."
                               " otherwise it should be passed in as a string.") from None
        elif isinstance(value, str):
            # since all the keys are in lower case, the passed in self._log_spec should be set to .lower()
            if value.lower() in list(self.__class__.LOG_SPECS.keys()):
                self._log_spec = self.LOG_SPECS[value.lower()]
            else:
                raise AttributeError(
                    f"log spec must be one of the following: {str(list(self.LOG_SPECS.keys()))[1:-1]}.")
        else:
            raise AttributeError("log spec value must be a string or a dict")


class _HandlerInitializer(_LogSpec):
    # noinspection PyTypeChecker
    def __init__(self):
        self._internal_logger: logging.Logger = None
        self.logger: logging.Logger = None
        self.formatter: logging.Formatter = None
        self.stream_formatter: logging.Formatter = None
        self.timestamp: str = None

    @property
    @abstractmethod
    def file_logger_levels(self):
        ...

    @property
    @abstractmethod
    def log_location(self) -> Path:
        ...

    @property
    @abstractmethod
    def project_name(self):
        ...

    def _add_filter_to_file_handler(self, handler: logging.FileHandler):
        """
        this is meant to be overwritten in a subclass to allow for filters
        to be added to file handlers without rewriting the entire method.

        Ex: new_filter = MyFilter()
        handler.addFilter(new_filter)
        :param handler:
        :type handler:
        :return:
        :rtype:
        """
        pass

    def _add_filter_to_stream_handler(self, handler: logging.StreamHandler):
        """
        this is meant to be overwritten in a subclass to allow for filters
        to be added to stream handlers without rewriting the entire method.

        Ex: new_filter = MyFilter()
        handler.addFilter(new_filter)

        :param handler:
        :type handler:
        :return:
        :rtype:
        """
        pass

    def make_file_handlers(self):
        """
        This method is used to create file handlers for the logger.
        It sets the logging level for each handler based on the file_logger_levels attribute.
        It also sets the log file location based on the logger level, project name, and timestamp.

        Parameters:
            self

        Returns:
            None

        Raises:
            None
        """
        self._internal_logger.info("creating file handlers for each logger level and log file location")
        for lvl in self.file_logger_levels:
            self.logger.setLevel(lvl)
            level_string = self.__class__.INT_TO_STR_LOGGER_LEVELS[self.logger.level]

            log_path = Path(self.log_location, '{}-{}-{}.log'.format(level_string,
                                                                     self.project_name, self.timestamp))

            # Create a file handler for the logger, and specify the log file location
            file_handler = logging.FileHandler(log_path)
            # Set the logging format for the file handler
            file_handler.setFormatter(self.formatter)
            file_handler.setLevel(self.logger.level)
            # doesn't do anything unless subclassed
            self._add_filter_to_file_handler(file_handler)

            # Add the file handlers to the loggers
            self.logger.addHandler(file_handler)

    def create_stream_handler(self, log_level_to_stream=logging.WARNING, **kwargs):
        """
        Creates and configures a StreamHandler for warning messages to print to the console.

        This method creates a StreamHandler and sets its logging format.
        The StreamHandler is then set to handle only warning level log messages.

        A one-time filter is added to the StreamHandler to ensure that warning messages are only printed to the console once.

        Finally, the StreamHandler is added to the logger.

        Note: This method assumes that `self.logger` and `self.formatter` are already defined.
        """

        if (log_level_to_stream not in self.__class__.INT_TO_STR_LOGGER_LEVELS
                and log_level_to_stream not in self.__class__.STR_TO_INT_LOGGER_LEVELS):
            raise ValueError(f"log_level_to_stream must be one of {list(self.__class__.STR_TO_INT_LOGGER_LEVELS)} or "
                             f"{list(self.__class__.INT_TO_STR_LOGGER_LEVELS)}, "
                             f"not {log_level_to_stream}")

        self._internal_logger.info(
            f"creating StreamHandler() for {logging.getLevelName(log_level_to_stream)} messages to print to console")

        use_one_time_filter = kwargs.get('use_one_time_filter', True)
        self._internal_logger.info(f"use_one_time_filter set to {use_one_time_filter}")

        # Create a stream handler for the logger
        stream_handler = kwargs.get('stream_handler_instance', logging.StreamHandler())
        # Set the logging format for the stream handler
        stream_handler.setFormatter(self.stream_formatter)
        stream_handler.setLevel(log_level_to_stream)
        if use_one_time_filter:
            # set the one time filter, so that log_level_to_stream messages will only be printed to the console once.
            one_time_filter = ConsoleOneTimeFilter()
            stream_handler.addFilter(one_time_filter)

        # doesn't do anything unless subclassed
        self._add_filter_to_stream_handler(stream_handler)

        # Add the stream handler to logger
        self.logger.addHandler(stream_handler)
        self._internal_logger.info(
            f"StreamHandler() for {logging.getLevelName(log_level_to_stream)} messages added. "
            f"{logging.getLevelName(log_level_to_stream)}s will be printed to console")
        if use_one_time_filter:
            self._internal_logger.info(f'Added filter {self.logger.handlers[-1].filters[0].name} to StreamHandler()')

    def _create_handler_instance(self, handler_to_create, handler_args, **kwargs):
        if handler_args is not None and isinstance(handler_to_create, type):
            instance = handler_to_create(**handler_args, **kwargs)
            self._internal_logger.info(f"{handler_to_create.__class__.__name__} handler created")
            self._internal_logger.debug(f"handler has the following args {dict(**handler_args, **kwargs)}")
        else:
            instance = handler_to_create
            self._internal_logger.info(f"instance of {handler_to_create.__class__.__name__} handler detected, moving to set up")
        return instance

    def create_other_handlers(self, handler_to_create: Union[type(logging.Handler), set[type(logging.Handler)]] = None,
                              handler_args: Optional[dict] = None, **kwargs):
        if handler_to_create and (callable(handler_to_create) or isinstance(handler_to_create, logging.Handler)):
            self._internal_logger.info(f"creating {handler_to_create.__class__.__name__} handler")
            instance = self._create_handler_instance(handler_to_create, handler_args, **kwargs)
            self._setup_other_handler(instance, **kwargs)
        else:
            self._internal_logger.debug(f"no other handlers created")

    def _setup_other_handler(self, handler_instance: logging.Handler, **kwargs):
        handler_instance.setLevel(kwargs.get('logging_level', self.logger.level))
        self._internal_logger.info(f"handler level set to {logging.getLevelName(handler_instance.level)}")

        handler_instance.setFormatter(kwargs.get('formatter', self.formatter))
        self._internal_logger.info(f"handler formatter set to {handler_instance.formatter.__class__.__name__}")

        self.logger.addHandler(handler_instance)
        self._internal_logger.info(f"{handler_instance.__class__.__name__} handler added")



from logging import Logger, getLevelName, StreamHandler


class _EasyLoggerCustomLogger(Logger):
    """
    A custom logger class that extends the standard Python Logger class, providing
    additional functionality such as selective message printing and message sanitization.

    Methods:
    --------
    _logger_should_print_normal_msg(self) -> bool:
        Determines if the logger should print normal messages based on the
        StreamHandler logging levels.

    sanitize_msg(msg: str) -> str: (staticmethod)
        Sanitizes the input message by encoding and decoding it using cp1250
        encoding, removing unsupported characters.

    _print_msg(self, msg: str, **kwargs) -> None:
        Prints the message to the console, if allowed by the logger's state and
        the method's arguments.

    _log(self, level: int, msg: str, args: tuple, exc_info=None, extra=None,
         stack_info=False, **kwargs) -> None:
        Logs a message at the specified logging level, optionally sanitizing
        and printing the message.

    info(self, msg: str, *args, **kwargs) -> None:
        Logs an informational message.

    warning(self, msg: str, *args, **kwargs) -> None:
        Logs a warning message.

    error(self, msg: str, *args, **kwargs) -> None:
        Logs an error message.

    debug(self, msg: str, *args, **kwargs) -> None:
        Logs a debug message.

    critical(self, msg: str, *args, **kwargs) -> None:
        Logs a critical message.
    """

    def _logger_should_print_normal_msg(self) -> bool:
        """
        Determines whether the logger should print normal messages based on the
        logging levels of its StreamHandler instances.

        :return: True if no StreamHandler is set to DEBUG or INFO level,
                 otherwise False.
        :rtype: bool
        """
        stream_handler_levels = [getLevelName(x.level) for x in
                                 self.handlers
                                 if type(x) is StreamHandler]
        if stream_handler_levels:
            if any([x for x in stream_handler_levels if x in ['DEBUG', 'INFO']]):
                return False
        return True

    @staticmethod
    def sanitize_msg(msg):
        """
        Sanitizes the input message by encoding it using 'cp1250' encoding with error ignoring
        and decoding it back.

        :param msg: The input message string to sanitize.
        :type msg: str
        :return: The sanitized message string.
        :rtype: str
        """
        if issubclass(msg.__class__, Exception):
            msg = str(msg)
        msg = msg.encode('cp1250', errors='ignore').decode('cp1250')
        return msg

    def _print_msg(self, msg, **kwargs):
        """
        :param msg: The message to be printed.
        :type msg: str
        :param kwargs: Optional keyword arguments, which include 'print_msg' to control whether the message should be printed.
        :type kwargs: dict
        :return: None
        :rtype: None
        """
        if kwargs.get('print_msg', False) and self._logger_should_print_normal_msg():
            print(msg)

    def _log(self, level, msg, args, exc_info=None, extra=None, stack_info=False, **kwargs):
        """
        :param level: The logging level specified for the log message.
        :type level: int
        :param msg: The message that needs to be logged.
        :type msg: str
        :param args: Arguments to be merged into the log message.
        :type args: tuple
        :param exc_info: Indicator or exception information for the log message. Can be a tuple, exception, or boolean.
        :type exc_info: Optional[Union[tuple, Exception, bool]]
        :param extra: Additional context information to include in the log record.
        :type extra: Optional[dict]
        :param stack_info: Whether stack information should be added to the log record.
        :type stack_info: bool
        :param kwargs: Additional keyword arguments to modify the log behavior.
        :type kwargs: dict
        :return: None
        :rtype: None
        """
        self._print_msg(msg, print_msg=kwargs.pop('print_msg', False))
        msg = self.sanitize_msg(msg)
        # noinspection PyProtectedMember
        super()._log(level, msg, args,
                     exc_info=exc_info,
                     extra=extra, stack_info=stack_info,
                     **kwargs)

    def info(self, msg, *args, **kwargs):
        super().info(msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        super().warning(msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        super().error(msg, *args, **kwargs)

    def debug(self, msg, *args, **kwargs):
        super().debug(msg, *args, **kwargs)

    def critical(self, msg, *args, **kwargs):
        super().critical(msg, *args, **kwargs)



from logging import Handler, StreamHandler
from pathlib import Path
from shutil import rmtree, copytree
from typing import Optional, Union
from zipfile import ZipFile

from EasyLoggerAJM import InvalidEmailMsgType, LogFilePrepError


class _BaseCustomEmailHandler(Handler):
    VALID_EMAIL_MSG_TYPES = []
    ERROR_TEMPLATE = "Error sending email: {error_msg}"

    def __init__(self, email_msg, logger_dir_path: Union[str, Path], recipient: Union[str, list], project_name='default_project_name',
                 **kwargs):
        super().__init__()
        self._email_msg = None
        self._recipient = None

        self.email_msg = email_msg  # kwargs.pop('email_msg', None)
        self.recipient = recipient
        self.project_name = project_name
        if isinstance(logger_dir_path, str):
            self.logger_dir_path = Path(logger_dir_path)
        elif isinstance(logger_dir_path, Path):
            self.logger_dir_path = logger_dir_path

        if not self.email_msg or not self.recipient:
            raise ValueError("email_msg and or recipient not provided.")

    @property
    def recipient(self) -> str:
        return self._recipient

    @recipient.setter
    def recipient(self, value: Union[str, list]):
        if not value:
            raise ValueError("recipient not provided.")
        if isinstance(value, list):
            self._recipient = ' ;'.join(value)
        else:
            self._recipient = value

    @property
    def email_msg(self):
        return self._email_msg

    @email_msg.setter
    def email_msg(self, value):
        if not value:
            raise ValueError("email_msg not provided.")

        if isinstance(value, tuple(self.VALID_EMAIL_MSG_TYPES)):
            if callable(value):
                self._email_msg = value()
            else:
                self._email_msg = value
        if (self.__class__.VALID_EMAIL_MSG_TYPES
                and len(self.__class__.VALID_EMAIL_MSG_TYPES) <= 0):
            raise InvalidEmailMsgType(
                valid_msg_types=self.__class__.VALID_EMAIL_MSG_TYPES,
                given_value=type(value))

    @staticmethod
    def _write_zip(zip_path: Union[Path, str] = None, copy_dest: Path = None):
        with ZipFile(zip_path, 'w') as zipf:
            for f in copy_dest.iterdir():
                if f.suffix == '.log':
                    zipf.write(f, arcname=f.name)

    def _prep_logfile_attachment(self, dir_path: Optional[Path] = None):
        if not dir_path:
            dir_path = Path(self.logger_dir_path.as_posix())
        if dir_path.is_dir():
            copy_dest = dir_path / 'copy_of_logfile'
            copytree(dir_path, copy_dest, dirs_exist_ok=True)
            zip_path = dir_path / 'copy_of_logfile.zip'

            self._write_zip(zip_path, copy_dest)
            return zip_path, copy_dest

    @staticmethod
    def _cleanup_logfile_zip(dir_path: Union[Path, str], zip_to_attach: Union[Path, str]):
        rmtree(dir_path, ignore_errors=True)
        zip_to_attach.unlink(missing_ok=True)


class OutlookEmailHandler(_BaseCustomEmailHandler):
    VALID_EMAIL_MSG_TYPES = []

    def __init_subclass__(cls, **kwargs):
        if not cls.VALID_EMAIL_MSG_TYPES:
            raise ValueError("VALID_EMAIL_MSG_TYPES not defined.")

    def _prepare_email(self, record):
        self.email_msg.To = self.recipient  # Replace with your recipient
        self.email_msg.Subject = f"{record.levelname} in {self.project_name}"
        self.email_msg.HTMLBody = self.format(record)

    def _prep_and_attach_logfile(self):
        zip_to_attach, copy_dir_path = self._prep_logfile_attachment()
        if zip_to_attach and zip_to_attach.is_file():
            self.email_msg.Attachments.Add(str(zip_to_attach.resolve()))
        return zip_to_attach, copy_dir_path

    def _send_and_cleanup_attachments(self, copy_dir_path, zip_to_attach, **kwargs):
        try:
            self.email_msg.Send()
            self._cleanup_logfile_zip(copy_dir_path, zip_to_attach)
        except Exception as e:
            err_string = self.__class__.ERROR_TEMPLATE.format(error_msg=e)
            print(err_string)
        finally:
            try:
                self._cleanup_logfile_zip(copy_dir_path, zip_to_attach)
            except UnboundLocalError:
                pass
            finally:
                self.email_msg.Attachments.Clear()
                self.email_msg.Send()

    def emit(self, record):
        self._prepare_email(record)
        try:
            zip_to_attach, copy_dir_path = self._prep_and_attach_logfile()
        except Exception as e:
            try:
                raise LogFilePrepError(e) from None
            finally:
                self._send_and_cleanup_attachments(copy_dir_path, zip_to_attach)


class StreamHandlerIgnoreExecInfo(StreamHandler):
    """
    A custom logging StreamHandler that temporarily suppresses exception information when emitting a log record.

    This handler is useful in scenarios where the exception information (`exc_info` and `exc_text`)
    should not be included in the StreamHandler output but needs to remain intact in the original log record.

    Methods:
        emit(record):
            Handles the log record emission by temporarily removing `exc_info` and `exc_text` attributes
            from the log record (if present) and restoring them after the emission. If `exc_info` is not
            present in the record, it simply calls the parent class's `emit` method.
    """
    def emit(self, record):
        """
        :param record: Log record to be processed and possibly emitted by the handler.
        :type record: logging.LogRecord
        :return: None
        :rtype: None
        """
        # Temporarily remove exc_info and exc_text for this handler
        if record.exc_info:
            # Save the original exc_info
            orig_exc_info = record.exc_info
            orig_exc_text = getattr(record, 'exc_text', None)
            record.exc_info = None
            record.exc_text = None
            try:
                # Call the parent class emit method
                super().emit(record)
            finally:
                # Restore the original exc_info back to the record
                record.exc_info = orig_exc_info
                record.exc_text = orig_exc_text
        else:
            super().emit(record)



from setuptools import setup
import re
project_name = 'EasyLoggerAJM'


def get_property(prop, project):
    result = re.search(r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop), open(project + '/_version.py').read())
    return result.group(1)


setup(
    name=project_name,
    version=get_property('__version__', project_name),
    packages=['EasyLoggerAJM'],
    install_requires=['ColorizerAJM'],
    url='https://github.com/amcsparron2793-Water/EasyLoggerAJM',
    download_url=f'https://github.com/amcsparron2793-Water/EasyLoggerAJM/archive/refs/tags/{get_property("__version__", project_name)}.tar.gz',
    keywords=['logging'],
    license='MIT License',
    author='Amcsparron',
    author_email='amcsparron@albanyny.gov',
    description='logger with already set up generalized file handlers'
)



from logging import Filter


class ConsoleOneTimeFilter(Filter):
    """
    ConsoleOneTimeFilter class filters log messages to only allow them to be logged once.
    :param logging.Filter: A class representing a log filter.
    :param name: A string indicating the name of the filter.
    :ivar logged_messages: A set to store logged messages.
    """
    def __init__(self, name="ConsoleWarnOneTime"):
        super().__init__(name)
        self.logged_messages = set()

    def filter(self, record):
        # We only log the message if it has not been logged before
        if record.msg not in self.logged_messages:
            self.logged_messages.add(record.msg)
            return True
        return False
